Object.defineProperty(exports, '__esModule', { value: true });

var fs = require('fs/promises');
var path = require('path');
var rollup = require('rollup');
var rollupPluginSwc3 = require('rollup-plugin-swc3');
var commonjs = require('@rollup/plugin-commonjs');
var shebang = require('rollup-plugin-preserve-shebang');
var json = require('@rollup/plugin-json');
var pluginNodeResolve = require('@rollup/plugin-node-resolve');
var replace = require('@rollup/plugin-replace');
var prettyBytes = require('pretty-bytes');
var module$1 = require('module');

function _interopDefault (e) { return e && e.__esModule ? e : { default: e }; }

var fs__default = /*#__PURE__*/_interopDefault(fs);
var path__default = /*#__PURE__*/_interopDefault(path);
var commonjs__default = /*#__PURE__*/_interopDefault(commonjs);
var shebang__default = /*#__PURE__*/_interopDefault(shebang);
var json__default = /*#__PURE__*/_interopDefault(json);
var replace__default = /*#__PURE__*/_interopDefault(replace);
var prettyBytes__default = /*#__PURE__*/_interopDefault(prettyBytes);

function chunkSizeCollector() {
    const sizes = new Map();
    function addSize(name, size) {
        sizes.set(name, size);
    }
    return {
        plugin: (cwd)=>{
            return {
                name: 'collect-sizes',
                augmentChunkHash () {
                // Do nothing, but use the hook to keep the plugin instance alive
                },
                renderChunk (code, chunk, options) {
                    const dir = options.dir || options.file && path__default.default.dirname(options.file);
                    let fileName = chunk.fileName;
                    if (dir) {
                        fileName = path__default.default.relative(cwd, path__default.default.join(dir, fileName));
                    }
                    addSize(fileName, code.length);
                    return null;
                }
            };
        },
        getSizeStats () {
            const sizeStats = [];
            sizes.forEach((size, name)=>{
                sizeStats.push([
                    name,
                    prettyBytes__default.default(size),
                    size
                ]);
            });
            return sizeStats;
        }
    };
}

function getTypings(pkg) {
    return pkg.types || pkg.typings;
}
function getDistPath(distPath, cwd) {
    return path.resolve(cwd, distPath);
}
function findExport(field) {
    if (!field) return;
    if (typeof field === 'string') return field;
    const value = field['.'] || field['import'] || field['module'] || field['default'];
    return findExport(value);
}
function parseExport(exportsCondition) {
    const paths = {};
    if (typeof exportsCondition === 'string') {
        paths.export = exportsCondition;
    } else {
        paths.main = paths.main || exportsCondition['require'] || exportsCondition['node'] || exportsCondition['default'];
        paths.module = paths.module || exportsCondition['module'];
        paths.export = findExport(exportsCondition);
    }
    return paths;
}
/**
 * Get package exports paths
 *
 * Example:
 *
 * ```json
 * {
 *  "exports": {
 *    ".": {
 *      "require": "./dist/index.cjs",
 *      "module": "./dist/index.esm.js",
 *      "default": "./dist/index.esm.js"
 *    },
 *    "./foo": {
 *      "require": "./dist/foo.cjs",
 *      "module": "./dist/foo.esm.js",
 *      "default": "./dist/foo.esm.js"
 *   }
 * }
 *
 * ```
 *
 * will be parsed to:
 *
 * ```js
 * {
 *   '.': {
 *     main: './dist/index.cjs',
 *     module: './dist/index.esm.js',
 *     export: './dist/index.esm.js'
 *   },
 *   './foo': {
 *     main: './dist/foo.cjs',
 *     module: './dist/foo.esm.js',
 *     export: './dist/foo.esm.js'
 *   }
 *
 *
 * pkg.main and pkg.module will be added to ['.'] if exists
 */ function getExportPaths(pkg) {
    const pathsMap = {};
    const mainExport = {};
    if (pkg.main) {
        mainExport.main = pkg.main;
    }
    if (pkg.module) {
        mainExport.module = pkg.module;
    }
    pathsMap['.'] = mainExport;
    const { exports: exportsConditions  } = pkg;
    if (exportsConditions) {
        if (typeof exportsConditions === 'string') {
            mainExport.export = exportsConditions;
        } else {
            const exportKeys = Object.keys(exportsConditions);
            if (exportKeys.some((key)=>key.startsWith('.'))) {
                exportKeys.forEach((subExport)=>{
                    pathsMap[subExport] = parseExport(exportsConditions[subExport]);
                });
            } else {
                Object.assign(mainExport, parseExport(exportsConditions));
            }
        }
    }
    pathsMap['.'] = mainExport;
    return pathsMap;
}
function getExportDist(pkg, cwd) {
    const paths = getExportPaths(pkg)['.'];
    const dist = [];
    if (paths.main) {
        dist.push({
            format: 'cjs',
            file: getDistPath(paths.main, cwd)
        });
    }
    if (paths.module) {
        dist.push({
            format: 'esm',
            file: getDistPath(paths.module, cwd)
        });
    }
    if (paths.export) {
        dist.push({
            format: 'esm',
            file: getDistPath(paths.export, cwd)
        });
    }
    // default fallback to output `dist/index.js` in default esm format
    if (dist.length === 0) {
        dist.push({
            format: 'esm',
            file: getDistPath('dist/index.js', cwd)
        });
    }
    return dist;
}
function getExportConditionDist(pkg, exportCondition, cwd) {
    const dist = [];
    // "exports": "..."
    if (typeof exportCondition === 'string') {
        dist.push({
            format: pkg.type === 'module' ? 'esm' : 'cjs',
            file: getDistPath(exportCondition, cwd)
        });
    } else {
        // "./<subexport>": { }
        const subExports = exportCondition;
        // Ignore json exports, like "./package.json"
        if (typeof subExports === 'string') {
            dist.push({
                format: 'esm',
                file: getDistPath(subExports, cwd)
            });
        } else {
            if (subExports.require) {
                dist.push({
                    format: 'cjs',
                    file: getDistPath(subExports.require, cwd)
                });
            }
            if (subExports.import) {
                dist.push({
                    format: 'esm',
                    file: getDistPath(subExports.import, cwd)
                });
            }
        }
    }
    return dist;
}

function asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator$3(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$3(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function exit(err) {
    logger.error(err);
    process.exit(1);
}
function getPackageMeta(cwd) {
    return _getPackageMeta.apply(this, arguments);
}
function _getPackageMeta() {
    _getPackageMeta = _async_to_generator$3(function*(cwd) {
        const pkgFilePath = path__default.default.resolve(cwd, 'package.json');
        let targetPackageJson = {};
        try {
            targetPackageJson = JSON.parse((yield fs__default.default.readFile(pkgFilePath, {
                encoding: 'utf-8'
            })));
        } catch (_) {}
        return targetPackageJson;
    });
    return _getPackageMeta.apply(this, arguments);
}
const logger = {
    log (arg) {
        console.log(arg);
    },
    warn (arg) {
        console.log('\x1b[33m' + arg + '\x1b[0m');
    },
    error (arg) {
        console.error('\x1b[31m' + arg + '\x1b[0m');
    }
};
function fileExists(filePath) {
    return _fileExists.apply(this, arguments);
}
function _fileExists() {
    _fileExists = _async_to_generator$3(function*(filePath) {
        try {
            yield fs__default.default.access(filePath);
            return true;
        } catch (err) {
            if (err.code === 'ENOENT') {
                return false;
            }
            throw err;
        }
    });
    return _fileExists.apply(this, arguments);
}
// . -> pkg name
// ./lite -> <pkg name>/lite
function getExportPath(pkg, cwd, exportName) {
    const name = pkg.name || path__default.default.basename(cwd);
    if (exportName === '.' || !exportName) return name;
    return path__default.default.join(name, exportName);
}
const isNotNull = (n)=>Boolean(n);
const SRC = 'src' // resolve from src/ directory
;
function resolveSourceFile(cwd, filename) {
    return path__default.default.resolve(cwd, SRC, filename);
}
// Map '.' -> './index.[ext]'
// Map './lite' -> './lite.[ext]'
// Return undefined if no match or if it's package.json exports
function getSourcePathFromExportPath(cwd, exportPath) {
    return _getSourcePathFromExportPath.apply(this, arguments);
}
function _getSourcePathFromExportPath() {
    _getSourcePathFromExportPath = _async_to_generator$3(function*(cwd, exportPath) {
        const exts = [
            'js',
            'cjs',
            'mjs',
            'jsx',
            'ts',
            'tsx'
        ];
        for (const ext of exts){
            // ignore package.json
            if (exportPath.endsWith('package.json')) return;
            if (exportPath === '.') exportPath = './index';
            const filename = resolveSourceFile(cwd, `${exportPath}.${ext}`);
            if (yield fileExists(filename)) {
                return filename;
            }
        }
        return;
    });
    return _getSourcePathFromExportPath.apply(this, arguments);
}

function asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator$2(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$2(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _extends$1() {
    _extends$1 = Object.assign || function(target) {
        for(var i = 1; i < arguments.length; i++){
            var source = arguments[i];
            for(var key in source){
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }
        return target;
    };
    return _extends$1.apply(this, arguments);
}
const minifyOptions = {
    compress: true,
    format: {
        comments: 'some',
        wrapFuncArgs: false,
        preserveAnnotations: true
    },
    mangle: {
        toplevel: true
    }
};
// This can also be passed down as stats from top level
const sizeCollector = chunkSizeCollector();
function getBuildEnv(envs) {
    if (!envs.includes('NODE_ENV')) {
        envs.push('NODE_ENV');
    }
    const envVars = envs.reduce((acc, key)=>{
        const value = process.env[key];
        if (typeof value !== 'undefined') {
            acc['process.env.' + key] = JSON.stringify(value);
        }
        return acc;
    }, {});
    return envVars;
}
function buildInputConfig(entry, pkg, options, cwd, { tsConfigPath , tsCompilerOptions  }, dtsOnly) {
    var _options_external;
    const externals = options.noExternal ? [] : [
        pkg.peerDependencies,
        pkg.dependencies,
        pkg.peerDependenciesMeta
    ].filter((n)=>Boolean(n)).map((o)=>Object.keys(o)).reduce((a, b)=>a.concat(b), []).concat(((_options_external = options.external) != null ? _options_external : []).concat(pkg.name ? [
        pkg.name
    ] : []));
    const { useTypescript , runtime , target: jscTarget , minify  } = options;
    const hasSpecifiedTsTarget = Boolean((tsCompilerOptions == null ? void 0 : tsCompilerOptions.target) && tsConfigPath);
    const sizePlugin = sizeCollector.plugin(cwd);
    const commonPlugins = [
        shebang__default.default(),
        sizePlugin
    ];
    const plugins = (dtsOnly ? [
        ...commonPlugins,
        useTypescript && require('rollup-plugin-dts').default({
            compilerOptions: _extends$1({}, tsCompilerOptions, {
                declaration: true,
                noEmit: false,
                noEmitOnError: true,
                emitDeclarationOnly: true,
                checkJs: false,
                declarationMap: false,
                skipLibCheck: true,
                preserveSymlinks: false,
                target: 'esnext',
                module: 'esnext',
                incremental: false,
                jsx: tsCompilerOptions.jsx || 'react'
            })
        })
    ] : [
        ...commonPlugins,
        replace__default.default({
            values: getBuildEnv(options.env || []),
            preventAssignment: true
        }),
        pluginNodeResolve.nodeResolve({
            preferBuiltins: runtime === 'node',
            extensions: [
                '.mjs',
                '.js',
                '.json',
                '.node',
                '.jsx'
            ]
        }),
        commonjs__default.default({
            include: /node_modules\//
        }),
        json__default.default(),
        rollupPluginSwc3.swc({
            include: /\.(m|c)?[jt]sx?$/,
            exclude: 'node_modules',
            tsconfig: tsConfigPath,
            jsc: _extends$1({}, !hasSpecifiedTsTarget && {
                target: jscTarget
            }, {
                loose: true,
                externalHelpers: false,
                parser: {
                    syntax: useTypescript ? 'typescript' : 'ecmascript',
                    [useTypescript ? 'tsx' : 'jsx']: true,
                    privateMethod: true,
                    classPrivateProperty: true,
                    exportDefaultFrom: true
                }
            }, minify && {
                minify: _extends$1({}, minifyOptions, {
                    sourceMap: options.sourcemap
                })
            }),
            sourceMaps: options.sourcemap,
            inlineSourcesContent: false
        })
    ]).filter(isNotNull);
    return {
        input: entry,
        external (id) {
            return externals.some((name)=>id === name || id.startsWith(name + '/'));
        },
        plugins,
        treeshake: {
            propertyReadSideEffects: false
        },
        onwarn (warning, warn) {
            const code = warning.code || '';
            // Some may not have types, like CLI binary
            if (dtsOnly && code === 'EMPTY_BUNDLE') return;
            if ([
                'MIXED_EXPORTS',
                'PREFER_NAMED_EXPORTS',
                'UNRESOLVED_IMPORT',
                'THIS_IS_UNDEFINED'
            ].includes(code)) return;
            // If the circular dependency warning is from node_modules, ignore it
            if (code === 'CIRCULAR_DEPENDENCY' && /Circular dependency: node_modules/.test(warning.message)) {
                return;
            }
            warn(warning);
        }
    };
}
function hasEsmExport(exportPaths, tsCompilerOptions) {
    let hasEsm = false;
    for(const key in exportPaths){
        const exportInfo = exportPaths[key];
        if (exportInfo.import || exportInfo.module) {
            hasEsm = true;
            break;
        }
    }
    return Boolean(hasEsm || (tsCompilerOptions == null ? void 0 : tsCompilerOptions.esModuleInterop));
}
function buildOutputConfigs(pkg, options, cwd, { tsCompilerOptions  }, dtsOnly) {
    const { format , exportCondition  } = options;
    const exportPaths = getExportPaths(pkg);
    // Add esm mark and interop helper if esm export is detected
    const useEsModuleMark = hasEsmExport(exportPaths, tsCompilerOptions);
    const typings = getTypings(pkg);
    const file = options.file && path.resolve(cwd, options.file);
    const dtsDir = typings ? path.dirname(path.resolve(cwd, typings)) : path.resolve(cwd, 'dist');
    // file base name without extension
    const name = file ? file.replace(new RegExp(`${path.extname(file)}$`), '') : undefined;
    const dtsFile = file ? name + '.d.ts' : (exportCondition == null ? void 0 : exportCondition.name) ? path.resolve(dtsDir, (exportCondition.name === '.' ? 'index' : exportCondition.name) + '.d.ts') : typings && path.resolve(cwd, typings);
    // If there's dts file, use `output.file`
    const dtsPathConfig = dtsFile ? {
        file: dtsFile
    } : {
        dir: dtsDir
    };
    return _extends$1({
        name: pkg.name || name
    }, dtsOnly ? dtsPathConfig : {
        file: file
    }, {
        format,
        exports: 'named',
        esModule: useEsModuleMark || 'if-default-prop',
        interop: 'auto',
        freeze: false,
        strict: false,
        sourcemap: options.sourcemap
    });
}
// build configs for each entry from package exports
function buildEntryConfig(pkg, bundleConfig, cwd, tsOptions, dtsOnly) {
    return _buildEntryConfig.apply(this, arguments);
}
function _buildEntryConfig() {
    _buildEntryConfig = _async_to_generator$2(function*(pkg, bundleConfig, cwd, tsOptions, dtsOnly) {
        const packageExports = pkg.exports || {};
        const configs = Object.keys(packageExports).map(/*#__PURE__*/ _async_to_generator$2(function*(entryExport) {
            const source = yield getSourcePathFromExportPath(cwd, entryExport);
            if (!source) return undefined;
            if (dtsOnly && !(tsOptions == null ? void 0 : tsOptions.tsConfigPath)) return;
            bundleConfig.exportCondition = {
                source,
                name: entryExport,
                export: packageExports[entryExport]
            };
            const entry = resolveSourceFile(cwd, source);
            const rollupConfig = buildConfig(entry, pkg, bundleConfig, cwd, tsOptions, dtsOnly);
            return rollupConfig;
        }));
        return (yield Promise.all(configs)).filter((n)=>Boolean(n));
    });
    return _buildEntryConfig.apply(this, arguments);
}
function buildConfig(entry, pkg, bundleConfig, cwd, tsOptions, dtsOnly) {
    var _options_exportCondition;
    const { file  } = bundleConfig;
    const useTypescript = Boolean(tsOptions.tsConfigPath);
    const options = _extends$1({}, bundleConfig, {
        useTypescript
    });
    const inputOptions = buildInputConfig(entry, pkg, options, cwd, tsOptions, dtsOnly);
    const outputExports = options.exportCondition ? getExportConditionDist(pkg, options.exportCondition.export, cwd) : getExportDist(pkg, cwd);
    let outputConfigs = [];
    // Generate dts job - single config
    if (dtsOnly) {
        outputConfigs = [
            buildOutputConfigs(pkg, _extends$1({}, bundleConfig, {
                format: 'es',
                useTypescript
            }), cwd, tsOptions, dtsOnly)
        ];
    } else {
        // multi outputs with specified format
        outputConfigs = outputExports.map((exportDist)=>{
            return buildOutputConfigs(pkg, _extends$1({}, bundleConfig, {
                file: exportDist.file,
                format: exportDist.format,
                useTypescript
            }), cwd, tsOptions, dtsOnly);
        });
        // CLI output option is always prioritized
        if (file) {
            var _outputExports_;
            const fallbackFormat = (_outputExports_ = outputExports[0]) == null ? void 0 : _outputExports_.format;
            outputConfigs = [
                buildOutputConfigs(pkg, _extends$1({}, bundleConfig, {
                    file,
                    format: bundleConfig.format || fallbackFormat,
                    useTypescript
                }), cwd, tsOptions, dtsOnly)
            ];
        }
    }
    return {
        input: inputOptions,
        output: outputConfigs,
        exportName: ((_options_exportCondition = options.exportCondition) == null ? void 0 : _options_exportCondition.name) || '.'
    };
}

function asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator$1(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep$1(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
let hasLoggedTsWarning = false;
function resolveTypescript(cwd) {
    let ts;
    const m = new module$1.Module('', undefined);
    m.paths = module$1.Module._nodeModulePaths(cwd);
    try {
        ts = m.require('typescript');
    } catch (_) {
        console.error(_);
        if (!hasLoggedTsWarning) {
            hasLoggedTsWarning = true;
            exit('Could not load TypeScript compiler. Try to install `typescript` as dev dependency');
        }
    }
    return ts;
}
function resolveTsConfig(cwd) {
    return _resolveTsConfig.apply(this, arguments);
}
function _resolveTsConfig() {
    _resolveTsConfig = _async_to_generator$1(function*(cwd) {
        let tsCompilerOptions = {};
        let tsConfigPath;
        const ts = resolveTypescript(cwd);
        tsConfigPath = path.resolve(cwd, 'tsconfig.json');
        if (yield fileExists(tsConfigPath)) {
            const basePath = tsConfigPath ? path.dirname(tsConfigPath) : cwd;
            const tsconfigJSON = ts.readConfigFile(tsConfigPath, ts.sys.readFile).config;
            tsCompilerOptions = ts.parseJsonConfigFileContent(tsconfigJSON, ts.sys, basePath).options;
        } else {
            tsConfigPath = undefined;
            return null;
        }
        return {
            tsCompilerOptions,
            tsConfigPath
        };
    });
    return _resolveTsConfig.apply(this, arguments);
}

function logSizeStats() {
    const stats = sizeCollector.getSizeStats();
    const maxLength = Math.max(...stats.map(([filename])=>filename.length));
    stats.forEach(([filename, prettiedSize])=>{
        const padding = ' '.repeat(maxLength - filename.length);
        const action = filename.endsWith('.d.ts') ? 'Typed' : 'Built';
        logger.log(` ✓  ${action} ${filename}${padding} - ${prettiedSize}`);
    });
}

function asyncGeneratorStep(gen, resolve, reject, _next, _throw, key, arg) {
    try {
        var info = gen[key](arg);
        var value = info.value;
    } catch (error) {
        reject(error);
        return;
    }
    if (info.done) {
        resolve(value);
    } else {
        Promise.resolve(value).then(_next, _throw);
    }
}
function _async_to_generator(fn) {
    return function() {
        var self = this, args = arguments;
        return new Promise(function(resolve, reject) {
            var gen = fn.apply(self, args);
            function _next(value) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "next", value);
            }
            function _throw(err) {
                asyncGeneratorStep(gen, resolve, reject, _next, _throw, "throw", err);
            }
            _next(undefined);
        });
    };
}
function _extends() {
    _extends = Object.assign || function(target) {
        for(var i = 1; i < arguments.length; i++){
            var source = arguments[i];
            for(var key in source){
                if (Object.prototype.hasOwnProperty.call(source, key)) {
                    target[key] = source[key];
                }
            }
        }
        return target;
    };
    return _extends.apply(this, arguments);
}
function _object_without_properties_loose(source, excluded) {
    if (source == null) return {};
    var target = {};
    var sourceKeys = Object.keys(source);
    var key, i;
    for(i = 0; i < sourceKeys.length; i++){
        key = sourceKeys[i];
        if (excluded.indexOf(key) >= 0) continue;
        target[key] = source[key];
    }
    return target;
}
function assignDefault(options, name, defaultValue) {
    if (!(name in options) || options[name] == null) {
        options[name] = defaultValue;
    }
}
function hasMultiEntryExport(pkg) {
    const packageExportsField = pkg.exports || {};
    if (typeof packageExportsField === 'string') return false;
    const exportKeys = (packageExportsField ? Object.keys(packageExportsField) : []).filter((key)=>key !== './package.json');
    return exportKeys.length > 0 && exportKeys.every((name)=>name.startsWith('.'));
}
function bundle(entryPath) {
    return _bundle.apply(this, arguments);
}
function _bundle() {
    _bundle = _async_to_generator(function*(entryPath, _param = {}) {
        var { cwd: _cwd  } = _param, options = _object_without_properties_loose(_param, [
            "cwd"
        ]);
        const cwd = path.resolve(process.cwd(), _cwd || '');
        assignDefault(options, 'format', 'es');
        assignDefault(options, 'minify', false);
        assignDefault(options, 'target', 'es2015');
        const pkg = yield getPackageMeta(cwd);
        const packageExportsField = pkg.exports || {};
        const isMultiEntries = hasMultiEntryExport(pkg);
        const tsConfig = yield resolveTsConfig(cwd);
        const hasTsConfig = Boolean(tsConfig == null ? void 0 : tsConfig.tsConfigPath);
        const defaultTsOptions = {
            tsConfigPath: tsConfig == null ? void 0 : tsConfig.tsConfigPath,
            tsCompilerOptions: (tsConfig == null ? void 0 : tsConfig.tsCompilerOptions) || {}
        };
        // Handle single entry file
        if (!isMultiEntries) {
            // Use specified string file path if possible, then fallback to the default behavior entry picking logic
            // e.g. "exports": "./dist/index.js" -> use "./index.<ext>" as entry
            entryPath = entryPath || (yield getSourcePathFromExportPath(cwd, '.')) || '';
        }
        const bundleOrWatch = (rollupConfig)=>{
            const { input , exportName  } = rollupConfig;
            const exportPath = getExportPath(pkg, cwd, exportName);
            // Log original entry file relative path
            const source = typeof input.input === 'string' ? path.relative(cwd, input.input) : exportPath;
            const buildMetadata = {
                source
            };
            if (options.watch) {
                return Promise.resolve(runWatch(rollupConfig, buildMetadata));
            }
            return runBundle(rollupConfig);
        };
        const hasSpecifiedEntryFile = entryPath ? (yield fileExists(entryPath)) && (yield fs__default.default.stat(entryPath)).isFile() : false;
        if (!hasSpecifiedEntryFile && !isMultiEntries) {
            const err = new Error(`Entry file \`${entryPath}\` is not existed`);
            err.name = 'NOT_EXISTED';
            return Promise.reject(err);
        }
        // has `types` field in package.json or has `types` exports in any export condition for multi-entries
        const hasTypings = !!getTypings(pkg) || typeof packageExportsField === 'object' && Array.from(Object.values(packageExportsField || {})).some((condition)=>condition.hasOwnProperty('types'));
        // Enable types generation if it's types field specified in package.json
        if (hasTypings) {
            options.dts = hasTypings;
        }
        let result;
        if (isMultiEntries) {
            const buildConfigs = yield buildEntryConfig(pkg, options, cwd, defaultTsOptions, false);
            const assetsJobs = buildConfigs.map((rollupConfig)=>bundleOrWatch(rollupConfig));
            const typesJobs = options.dts ? (yield buildEntryConfig(pkg, options, cwd, defaultTsOptions, true)).map((rollupConfig)=>bundleOrWatch(rollupConfig)) : [];
            result = yield Promise.all(assetsJobs.concat(typesJobs));
        } else {
            // Generate types
            if (hasTsConfig && options.dts) {
                yield bundleOrWatch(buildConfig(entryPath, pkg, options, cwd, defaultTsOptions, true));
            }
            const rollupConfig = buildConfig(entryPath, pkg, options, cwd, defaultTsOptions, false);
            result = yield bundleOrWatch(rollupConfig);
        }
        logSizeStats();
        return result;
    });
    return _bundle.apply(this, arguments);
}
function runWatch({ input , output  }, metadata) {
    const watchOptions = [
        _extends({}, input, {
            output: output,
            watch: {
                exclude: [
                    'node_modules/**'
                ]
            }
        })
    ];
    const watcher = rollup.watch(watchOptions);
    watcher.on('event', (event)=>{
        switch(event.code){
            case 'ERROR':
                {
                    return onError(event.error);
                }
            case 'START':
                {
                    logger.log(`Start building ${metadata.source} ...`);
                    break;
                }
            case 'END':
                {
                    break;
                }
            default:
                return;
        }
    });
    return watcher;
}
function runBundle({ input , output  }) {
    return rollup.rollup(input).then((bundle)=>{
        const writeJobs = output.map((options)=>bundle.write(options));
        return Promise.all(writeJobs);
    }, onError);
}
function onError(error) {
    if (!error) return;
    // logging source code in format
    if (error.frame) {
        process.stderr.write(error.frame + '\n');
    }
    // filter out the rollup plugin error information such as loc/frame/code...
    const err = new Error(error.message);
    err.stack = error.stack;
    throw err;
}

exports.bundle = bundle;
